package main

import (
	"flag"
	"fmt"
	"log"
	"net/http"
	"os"
	"path"
	"strings"

	fronttime "srs.epita.fr/fic-server/frontend/time"
	"srs.epita.fr/fic-server/settings"
)

func main() {
	var bind = flag.String("bind", "127.0.0.1:8080", "Bind port/socket")
	var prefix = flag.String("prefix", "/", "Request path prefix to strip (from proxy)")
	var teamsDir = flag.String("teams", "./TEAMS/", "Base directory where find existing teams")
	flag.StringVar(&settings.SettingsDir, "settings", settings.SettingsDir, "Base directory where read settings")
	flag.StringVar(&startedFile, "startedFile", startedFile, "Path to the file to create/remove whether or not the challenge is running")
	flag.StringVar(&SubmissionDir, "submission", "./submissions/", "Base directory where save submissions")
	flag.Parse()

	log.SetPrefix("[frontend] ")

	startedFile = path.Clean(startedFile)
	SubmissionDir = path.Clean(SubmissionDir)
	TmpSubmissionDir = path.Join(SubmissionDir, ".tmp")

	log.Println("Creating submission directory...")
	if _, err := os.Stat(TmpSubmissionDir); os.IsNotExist(err) {
		if err := os.MkdirAll(TmpSubmissionDir, 0700); err != nil {
			log.Fatal("Unable to create submission directory: ", err)
		}
	}

	*prefix = strings.TrimRight(*prefix, "/")

	// Load configuration
	settings.LoadAndWatchSettings(path.Join(settings.SettingsDir, settings.SettingsFile), reloadSettings)

	// Register handlers
	http.Handle(fmt.Sprintf("%s/chname/", *prefix), http.StripPrefix(fmt.Sprintf("%s/chname/", *prefix), submissionTeamChecker{"name change", ChNameHandler, *teamsDir}))
	http.Handle(fmt.Sprintf("%s/openhint/", *prefix), http.StripPrefix(fmt.Sprintf("%s/openhint/", *prefix), submissionTeamChecker{"opening hint", HintHandler, *teamsDir}))
	http.Handle(fmt.Sprintf("%s/registration/", *prefix), http.StripPrefix(fmt.Sprintf("%s/registration/", *prefix), submissionChecker{"registration", RegistrationHandler}))
	http.Handle(fmt.Sprintf("%s/resolution/", *prefix), http.StripPrefix(fmt.Sprintf("%s/resolution/", *prefix), ResolutionHandler{}))
	http.Handle(fmt.Sprintf("%s/submission/", *prefix), http.StripPrefix(fmt.Sprintf("%s/submission/", *prefix), submissionTeamChecker{"submission", SubmissionHandler, *teamsDir}))
	http.Handle(fmt.Sprintf("%s/time.json", *prefix), http.StripPrefix(*prefix, fronttime.TimeHandler{}))

	// Serve pages
	log.Println(fmt.Sprintf("Ready, listening on %s", *bind))
	if err := http.ListenAndServe(*bind, nil); err != nil {
		log.Fatal("Unable to listen and serve: ", err)
	}
}
