package fic

import (
	"time"
)

var UnlockedChallenges bool

type Team struct {
	Id          int64  `json:"id"`
	Name        string `json:"name"`
	Color       uint32 `json:"color"`
}

// Access functions

func GetTeams() ([]Team, error) {
	if rows, err := DBQuery("SELECT id_team, name, color FROM teams"); err != nil {
		return nil, err
	} else {
		defer rows.Close()

		var teams = make([]Team, 0)
		for rows.Next() {
			var t Team
			if err := rows.Scan(&t.Id, &t.Name, &t.Color); err != nil {
				return nil, err
			}
			teams = append(teams, t)
		}
		if err := rows.Err(); err != nil {
			return nil, err
		}

		return teams, nil
	}
}

func GetTeam(id int64) (Team, error) {
	var t Team
	if err := DBQueryRow("SELECT id_team, name, color FROM teams WHERE id_team = ?", id).Scan(&t.Id, &t.Name, &t.Color); err != nil {
		return t, err
	}

	return t, nil
}

func GetTeamBySerial(serial int64) (Team, error) {
	var t Team
	if err := DBQueryRow("SELECT T.id_team, T.name, T.color FROM certificates C INNER JOIN teams T ON T.id_team = C.id_team WHERE id_cert = ?", serial).Scan(&t.Id, &t.Name, &t.Color); err != nil {
		return t, err
	}

	return t, nil
}


// CRUD method

func CreateTeam(name string, color uint32) (Team, error) {
	if res, err := DBExec("INSERT INTO teams (name, color) VALUES (?, ?)", name, color); err != nil {
		return Team{}, err
	} else if tid, err := res.LastInsertId(); err != nil {
		return Team{}, err
	} else {
		return Team{tid, name, color}, nil
	}
}

func (t Team) Update() (int64, error) {
	if res, err := DBExec("UPDATE teams SET name = ?, color = ? WHERE id_team = ?", t.Name, t.Color, t.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}

func (t Team) Delete() (int64, error) {
	if _, err := DBExec("DELETE FROM team_members WHERE id_team = ?", t.Id); err != nil {
		return 0, err
	} else if res, err := DBExec("DELETE FROM teams WHERE id_team = ?", t.Id); err != nil {
		return 0, err
	} else if nb, err := res.RowsAffected(); err != nil {
		return 0, err
	} else {
		return nb, err
	}
}


// Exercice related functions

func (t Team) HasAccess(e Exercice) bool {
	if e.Depend == nil || UnlockedChallenges {
		return true
	} else {
		ed := Exercice{}
		ed.Id = *e.Depend
		s, _, _ := t.HasSolved(ed)
		return s
	}
}

func NbTry(t *Team, e Exercice) int {
	var cnt *int

	if t != nil {
		DBQueryRow("SELECT COUNT(*) FROM exercice_tries WHERE id_team = ? AND id_exercice = ?", t.Id, e.Id).Scan(&cnt)
	} else {
		DBQueryRow("SELECT COUNT(*) FROM exercice_tries WHERE id_exercice = ?", e.Id).Scan(&cnt)
	}

	if cnt == nil {
		return 0
	} else {
		return *cnt
	}
}

func (t Team) HasHint(h EHint) (bool) {
	var tm *time.Time
	DBQueryRow("SELECT MIN(time) FROM team_hints WHERE id_team = ? AND id_hint = ?", t.Id, h.Id).Scan(&tm)
	return tm != nil
}

func (t Team) OpenHint(h EHint) (error) {
	_, err := DBExec("INSERT INTO team_hints (id_team, id_hint, time) VALUES (?, ?, ?)", t.Id, h.Id, time.Now())
	return err
}

func (t Team) CountTries(e Exercice) (int64, time.Time) {
	var nb *int64
	var tm *time.Time
	if DBQueryRow("SELECT COUNT(id_exercice), MAX(time) FROM exercice_tries WHERE id_team = ? AND id_exercice = ?", t.Id, e.Id).Scan(&nb, &tm); tm == nil {
		return 0, time.Unix(0, 0)
	} else if nb == nil {
		return 0, *tm
	} else {
		return *nb, *tm
	}
}

func (t Team) LastTryDist(e Exercice) int64 {
	var nb *int64
	if DBQueryRow("SELECT nbdiff FROM exercice_tries WHERE id_team = ? AND id_exercice = ? ORDER BY time DESC LIMIT 1", t.Id, e.Id).Scan(&nb); nb == nil {
		return 0
	} else {
		return *nb
	}
}

func (t Team) HasSolved(e Exercice) (bool, time.Time, int64) {
	var nb *int64
	var tm *time.Time
	if DBQueryRow("SELECT MIN(time) FROM exercice_solved WHERE id_team = ? AND id_exercice = ?", t.Id, e.Id).Scan(&tm); tm == nil {
		return false, time.Unix(0, 0), 0
	} else if DBQueryRow("SELECT COUNT(id_exercice) FROM exercice_solved WHERE id_exercice = ? AND time < ?", e.Id, tm).Scan(&nb); nb == nil {
		return true, *tm, 0
	} else {
		return true, *tm, *nb + 1
	}
}

func IsSolved(e Exercice) (int, time.Time) {
	var nb *int
	var tm *time.Time
	if DBQueryRow("SELECT COUNT(id_exercice), MIN(time) FROM exercice_solved WHERE id_exercice = ?", e.Id).Scan(&nb, &tm); nb == nil || tm == nil {
		return 0, time.Time{}
	} else {
		return *nb, *tm
	}
}

func (t Team) HasPartiallySolved(k Key) (*time.Time) {
	var tm *time.Time
	DBQueryRow("SELECT MIN(time) FROM key_found WHERE id_team = ? AND id_key = ?", t.Id, k.Id).Scan(&tm)
	return tm
}

func (t Team) HasPartiallyRespond(m MCQ) (*time.Time) {
	var tm *time.Time
	DBQueryRow("SELECT MIN(time) FROM mcq_found WHERE id_team = ? AND id_mcq = ?", t.Id, m.Id).Scan(&tm)
	return tm
}
